from typing import Optional, List
from prometheus_client import Histogram, Counter
import os
from loguru import logger


class RequestOperationMetrics:
    """
    如果启动prometheus则使用此类
    """

    def __init__(self, buckets: Optional[List[float]] = None):
        default_buckets = [0.1, 0.5, 1, 3, 5]
        self.buckets = buckets if buckets is not None else default_buckets
        self.pid = os.getpid()

    def add(self, key, cost): ...

    def add_error(self, key): ...


class PrometheusRequestOperationMetrics(RequestOperationMetrics):
    """
    算法接口中算法操作的耗时统计
    """

    def __init__(self, buckets: Optional[List[float]] = None):
        super().__init__(buckets=buckets)

        self.ops_duration = Histogram(
            "operation_duration_seconds",
            "Operation duration in seconds",
            ["operation", "pid"],
            buckets=self.buckets,
        )
        self.ops_error_cnt = Counter(
            "operation_errors_total",
            "The total number of errors in operations.",
            ["operation", "pid"],
        )

    def add(self, key, cost):
        self.ops_duration.labels(operation=key, pid=self.pid).observe(cost)

    def add_error(self, key):
        self.ops_error_cnt.labels(operation=key, pid=self.pid).inc(1)


def request_operation_metrics_factory(buckets: Optional[List[float]] = None) -> RequestOperationMetrics:
    is_use_prometheus: str = os.environ.get("ignore_request_operation_metrics", "True")
    if is_use_prometheus.lower() == "true":
        logger.debug("Use prometheus RequestOperationMetrics")
        return PrometheusRequestOperationMetrics(buckets=buckets)
    else:
        logger.debug("Use empty RequestOperationMetrics")
        return RequestOperationMetrics(buckets=buckets)



import asyncio
import os

import psutil
from prometheus_client import Gauge


class SystemMetricsMonitor:
    def __init__(self):
        self.CPU_GAUGE = Gauge(
            "cpu_usage_percent",
            "CPU usage in percent",
            ["pid"],
            multiprocess_mode="all",
        )

        self.MEMORY_GAUGE = Gauge(
            "memory_usage_mb", "Memory usage in MB", ["pid"], multiprocess_mode="all"
        )

        self.pid = os.getpid()
        self.process = psutil.Process(self.pid)

    async def run_monitor(self, interval_sec: int = 10):
        async def work():
            while True:
                cpu_usage = self.process.cpu_percent()
                self.CPU_GAUGE.labels(pid=self.pid).set(cpu_usage)

                memory_info = self.process.memory_info()
                memory_usage = memory_info.rss / (1024 * 1024)
                self.MEMORY_GAUGE.labels(pid=self.process).set(memory_usage)

                await asyncio.sleep(interval_sec)

        asyncio.create_task(work())



from .system_metrics import SystemMetricsMonitor
from .request_metrics import RequestOperationMetrics
from .http_metrics import RequestMonitorMiddleware
from .prometheus_context import PrometheusContext


__all__ = [
    "SystemMetricsMonitor",
    "RequestOperationMetrics",
    "RequestMonitorMiddleware",
    "PrometheusContext"
]